#! /bin/sh

awk '
BEGIN { 
	FS = "|"
}
$4 == "DR" {
	if (NF != 13 && NF != 15) {
		print "Bad record line", NR, $0 | "cat 1>&2"
		next
	}
	if ($5 == "positive")
		good_drs ++
	else if ($5 == "negative")
		bad_drs ++
	else print "Unknown DR type", $4, NR, $0 | "cat 1>&2"
}
$4 == "ok" || $4 == "ok-testfailed" || $4 == "failed" {
	if (NF != 13) { 
		print "Bad record line NR", $0 | "cat 1>&2"
		next
	}
	split ($3, parts, ":")
	if (NR == 1) {
		first_month = $1; first_day = $2
		first_hour = parts[1]
		first_min = parts[2]
		first_sec = parts[3]
	}
	last_month = $1; last_day = $2
	last_hour = parts[1]
	last_min = parts[2]
	last_sec = parts[3]

	if ($4 != "ok") {
		failed ++
		failure_addr[$4] ++
		if (fm[$5] == 0) {
			failure_msgs[$4] ++
			fm[$5] = 1
		}
	}

	total_addr ++
	if (im[$5] == 0) {
		total_msgs ++
		total_size += $8
		im[$5] = 1
		inchan_size[$6] += $8
		if (ih[$6] == 0) {
			total_inmta ++
			inchan_nhosts[$6] ++
			ih[$6] = 1
		}
		inchan_nmsgs[$6] ++
	}
	inchan_naddr[$6] ++

	if (om[$5 $7] == 0) {
		om[$5 $7] = 1
		outchan_size[$7] += $8
		if (oh[$7] == 0) {
			total_outmta ++
			outchan_nhosts[$7] ++
			oh[$7] = 1
		}
		outchan_nmsgs[$7] ++
	}
	outchan_naddr[$7] ++;

	
	cp = $6 " -> " $7
	chpair_naddr[cp] ++
	if (cm[$5 cp] == 0) {
		cm[$5 cp] = 1
		chpair_size[cp] += $8
		chpair_nmsgs[cp] ++
	}
}
END {
	printf "SUMMARY for period %d/%d %d:%02d:%02d to %d/%d %d:%02d:%02d\n", \
		first_month, first_day, first_hour, first_min, first_sec, \
		last_month, last_day, last_hour, last_min, last_sec

	printf "%-20s: %d\n", "Messages", total_msgs
	printf "%-20s: %d\n", "Addresses", total_addr
	printf "%-20s: %d\n", "Bytes", total_size
	
	if (total_size < 1000)
		printf "%d\n", total_size
	printf "%-20s: %d\n", "Inbound MTAs", total_inmta
	printf "%-20s: %d\n", "Outbound MTAs", total_outmta
	if (good_drs)
		printf "%-20s: %d\n", "Positive DRs", good_drs
	if (bad_drs)
		printf "%-20s: %d\n", "Nagative Drs", bad_drs
	if (failed) {
		for (i in failure_addr)
			printf "%-20s: %d Addrs %d Msgs\n", "Failure - " i, \
				failure_addr[i], failure_msgs[i]
	}
	print "\n"

	print "Inbound Channel Totals"
	title =  "Channel"
	maxw = length(title)
	for (i in inchan_naddr)
		if (length(i) > maxw)
			maxw = length(i)
	format = "%-" maxw "s %8d %8d %12d\n"
	printf "%-" maxw "s %8s %8s %12s\n", title, \
		"Msgs", "Addr", "Bytes", "MTAs"
	for (i in inchan_naddr)
		printf format, i, \
			inchan_nmsgs[i], inchan_naddr[i], \
			inchan_size[i],	inchan_nhosts[i]

	print ""; print "";
	print "Outbound Channel Totals"
	title = "Channel"
	maxw = length(title)
	if (length(i) > maxw)
		maxw = length(i)
	
	printf "%-" maxw "s %8s %8s %12s\n", title,\
		"Msgs", "Addr", "Bytes", "MTAs"

	for (i in outchan_naddr)
		printf "%-" maxw "s %8d %8d %12d\n", i, \
			outchan_nmsgs[i], outchan_naddr[i], \
			outchan_size[i], outchan_nhosts[i]

	print ""; print ""
	print "Channel Pair Totals"
	title = "Channel"
	maxw = length(title)

	for (i in chpair_naddr)
		if (length(i) > maxw)
			maxw = length(i)
	printf "%-" maxw "s %8s %8s %12s\n", title, \
			"Msgs", "Addrs", "Bytes"
	for (i in chpair_naddr)
		printf "%-" maxw "s %8d %8d %12d\n", i, \
			chpair_nmsgs[i], chpair_naddr[i], \
			chpair_size[i]

}' $*
